@@ -1,7 +1,7 @@
 """This module defines ``AsyncQuery`` class."""
 
 from collections.abc import Sequence
-from typing import Any, Generic, Literal, TypeVar, overload
+from typing import Any, Generic, Literal, overload
 
 from sqlalchemy.engine import Result, Row, ScalarResult
 from sqlalchemy.orm import joinedload, selectinload, subqueryload
@@ -11,16 +11,20 @@ from sqlalchemy.sql._typing import _ColumnsClauseArgument
 from sqlalchemy.sql.base import ExecutableOption
 from sqlalchemy.sql.elements import ColumnElement
 from sqlalchemy.sql.functions import func
-from typing_extensions import Self
 
 from .exceptions import RelationError
 from .session import SessionMixin
-from .smart_query import ColumnExpressionOrStrLabelArgument, SmartQueryMixin
-
-_T = TypeVar('_T')
+from .smart_query import SmartQueryMixin
+from .types import (
+    ColumnExpressionOrStrLabelArgument,
+    EagerLoadPath,
+    EagerSchema,
+    Self,
+    T,
+)
 
 
-class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
+class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[T]):
     """Async wrapper for ``sqlalchemy.sql.Select``.
 
     Provides a set of helper methods for
@@ -113,13 +117,13 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         async with self.AsyncSession() as session:
             return await session.execute(self.query)
 
-    async def scalars(self) -> ScalarResult[_T]:
+    async def scalars(self) -> ScalarResult[T]:
         """Returns a ``sqlalchemy.engine.ScalarResult`` instance
         containing all results.
 
         Returns
         -------
-        ScalarResult[_T]
+        ScalarResult[T]
             Result instance containing all scalars.
 
         Examples
@@ -150,10 +154,10 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         return (await self.execute()).scalars()
 
     @overload
-    async def first(self) -> _T | None: ...
+    async def first(self) -> T | None: ...
 
     @overload
-    async def first(self, scalar: Literal[True]) -> _T | None: ...
+    async def first(self, scalar: Literal[True]) -> T | None: ...
 
     @overload
     async def first(
@@ -161,9 +165,7 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
     ) -> Row[tuple[Any, ...]] | None: ...
 
     @overload
-    async def first(
-        self, scalar: bool
-    ) -> _T | Row[tuple[Any, ...]] | None: ...
+    async def first(self, scalar: bool) -> T | Row[tuple[Any, ...]] | None: ...
 
     async def first(self, scalar: bool = True):
         """Fetches the first row or ``None`` if no results are found.
@@ -221,16 +223,16 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         return (await self.execute()).first()
 
     @overload
-    async def one(self) -> _T: ...
+    async def one(self) -> T: ...
 
     @overload
-    async def one(self, scalar: Literal[True]) -> _T: ...
+    async def one(self, scalar: Literal[True]) -> T: ...
 
     @overload
     async def one(self, scalar: Literal[False]) -> Row[tuple[Any, ...]]: ...
 
     @overload
-    async def one(self, scalar: bool) -> _T | Row[tuple[Any, ...]]: ...
+    async def one(self, scalar: bool) -> T | Row[tuple[Any, ...]]: ...
 
     async def one(self, scalar: bool = True):
         """Fetches one row or raises a ``sqlalchemy.exc.NoResultFound``
@@ -309,10 +311,10 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         return (await self.execute()).one()
 
     @overload
-    async def one_or_none(self) -> _T | None: ...
+    async def one_or_none(self) -> T | None: ...
 
     @overload
-    async def one_or_none(self, scalar: Literal[True]) -> _T | None: ...
+    async def one_or_none(self, scalar: Literal[True]) -> T | None: ...
 
     @overload
     async def one_or_none(
@@ -322,7 +324,7 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
     @overload
     async def one_or_none(
         self, scalar: bool
-    ) -> _T | Row[tuple[Any, ...]] | None: ...
+    ) -> T | Row[tuple[Any, ...]] | None: ...
 
     async def one_or_none(self, scalar: bool = True):
         """Fetches one row or ``None`` if no results are found.
@@ -401,10 +403,10 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         return (await self.execute()).one_or_none()
 
     @overload
-    async def all(self) -> Sequence[_T]: ...
+    async def all(self) -> Sequence[T]: ...
 
     @overload
-    async def all(self, scalars: Literal[True]) -> Sequence[_T]: ...
+    async def all(self, scalars: Literal[True]) -> Sequence[T]: ...
 
     @overload
     async def all(
@@ -414,7 +416,7 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
     @overload
     async def all(
         self, scalars: bool
-    ) -> Sequence[_T] | Sequence[Row[tuple[Any, ...]]]: ...
+    ) -> Sequence[T] | Sequence[Row[tuple[Any, ...]]]: ...
 
     async def all(self, scalars: bool = True):
         """Fetches all rows.
@@ -429,7 +431,7 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
 
         Returns
         -------
-        Sequence[_T]
+        Sequence[T]
             Instances (scalars).
         Sequence[Row[tuple[Any, ...]]]
             Rows.
@@ -490,10 +492,10 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         return (await self.execute()).scalars().one()
 
     @overload
-    async def unique(self) -> ScalarResult[_T]: ...
+    async def unique(self) -> ScalarResult[T]: ...
 
     @overload
-    async def unique(self, scalars: Literal[True]) -> ScalarResult[_T]: ...
+    async def unique(self, scalars: Literal[True]) -> ScalarResult[T]: ...
 
     @overload
     async def unique(
@@ -503,7 +505,7 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
     @overload
     async def unique(
         self, scalars: bool
-    ) -> ScalarResult[_T] | Result[tuple[Any, ...]]: ...
+    ) -> ScalarResult[T] | Result[tuple[Any, ...]]: ...
 
     async def unique(self, scalars: bool = True):
         """Similar to ``scalars()`` but applies unique filtering
@@ -528,7 +530,7 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
 
         Returns
         -------
-        ScalarResult[_T]
+        ScalarResult[T]
             Result instance containing all scalars.
         Result[tuple[Any, ...]]
             Result instance containing all rows.
@@ -560,10 +562,10 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         return (await self.execute()).unique()
 
     @overload
-    async def unique_first(self) -> _T | None: ...
+    async def unique_first(self) -> T | None: ...
 
     @overload
-    async def unique_first(self, scalar: Literal[True]) -> _T | None: ...
+    async def unique_first(self, scalar: Literal[True]) -> T | None: ...
 
     @overload
     async def unique_first(
@@ -573,7 +575,7 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
     @overload
     async def unique_first(
         self, scalar: bool
-    ) -> _T | Row[tuple[Any, ...]] | None: ...
+    ) -> T | Row[tuple[Any, ...]] | None: ...
 
     async def unique_first(self, scalar: bool = True):
         """Similar to ``first()`` but applies unique filtering to the
@@ -592,10 +594,10 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         return (await self.unique(scalar)).first()
 
     @overload
-    async def unique_one(self) -> _T: ...
+    async def unique_one(self) -> T: ...
 
     @overload
-    async def unique_one(self, scalar: Literal[True]) -> _T: ...
+    async def unique_one(self, scalar: Literal[True]) -> T: ...
 
     @overload
     async def unique_one(
@@ -603,7 +605,7 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
     ) -> Row[tuple[Any, ...]]: ...
 
     @overload
-    async def unique_one(self, scalar: bool) -> _T | Row[tuple[Any, ...]]: ...
+    async def unique_one(self, scalar: bool) -> T | Row[tuple[Any, ...]]: ...
 
     async def unique_one(self, scalar: bool = True):
         """Similar to ``one()`` but applies unique filtering to the
@@ -622,10 +624,10 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         return (await self.unique(scalar)).one()
 
     @overload
-    async def unique_one_or_none(self) -> _T | None: ...
+    async def unique_one_or_none(self) -> T | None: ...
 
     @overload
-    async def unique_one_or_none(self, scalar: Literal[True]) -> _T | None: ...
+    async def unique_one_or_none(self, scalar: Literal[True]) -> T | None: ...
 
     @overload
     async def unique_one_or_none(
@@ -635,7 +637,7 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
     @overload
     async def unique_one_or_none(
         self, scalar: bool
-    ) -> _T | Row[tuple[Any, ...]] | None: ...
+    ) -> T | Row[tuple[Any, ...]] | None: ...
 
     async def unique_one_or_none(self, scalar: bool = True):
         """Similar to ``one_or_none()`` but applies
@@ -655,10 +657,10 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         return (await self.unique(scalar)).one_or_none()
 
     @overload
-    async def unique_all(self) -> Sequence[_T]: ...
+    async def unique_all(self) -> Sequence[T]: ...
 
     @overload
-    async def unique_all(self, scalars: Literal[True]) -> Sequence[_T]: ...
+    async def unique_all(self, scalars: Literal[True]) -> Sequence[T]: ...
 
     @overload
     async def unique_all(
@@ -668,7 +670,7 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
     @overload
     async def unique_all(
         self, scalars: bool
-    ) -> Sequence[_T] | Sequence[Row[tuple[Any, ...]]]: ...
+    ) -> Sequence[T] | Sequence[Row[tuple[Any, ...]]]: ...
 
     async def unique_all(self, scalars: bool = True):
         """Similar to ``all()`` but applies unique filtering to the
@@ -1259,10 +1261,7 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         return self.limit(top)
 
     def join(
-        self,
-        *paths: InstrumentedAttribute[Any]
-        | tuple[InstrumentedAttribute[Any], bool],
-        model: type[_T] | None = None,
+        self, *paths: EagerLoadPath, model: type[T] | None = None
     ) -> Self:
         """Joined eager loading using LEFT OUTER JOIN.
 
@@ -1274,9 +1273,9 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
 
         Parameters
         ----------
-        *paths : InstrumentedAttribute[Any] | tuple[InstrumentedAttribute[Any], bool]
+        *paths : EagerLoadPath
             Relationship attributes to join.
-        model : type[_T] | None, optional
+        model : type[T] | None, optional
             If given, checks that each path belongs to this model,
             by default None.
 
@@ -1337,10 +1336,7 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         )
 
     def with_subquery(
-        self,
-        *paths: InstrumentedAttribute[Any]
-        | tuple[InstrumentedAttribute[Any], bool],
-        model: type[_T] | None = None,
+        self, *paths: EagerLoadPath, model: type[T] | None = None
     ) -> Self:
         """Subqueryload or Selectinload eager loading.
 
@@ -1387,9 +1383,9 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
 
         Parameters
         ----------
-        *paths : InstrumentedAttribute[Any] | tuple[InstrumentedAttribute[Any], bool]
+        *paths : EagerLoadPath
             Relationship attributes to load.
-        model : type[_T] | None, optional
+        model : type[T] | None, optional
             If given, checks that each path belongs to this model,
             by default None.
 
@@ -1462,13 +1458,7 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         """
         return self._apply_eager_loading_options(*paths, model=model)
 
-    def with_schema(
-        self,
-        schema: dict[
-            InstrumentedAttribute[Any],
-            str | tuple[str, dict[InstrumentedAttribute[Any], Any]] | dict,
-        ],
-    ) -> Self:
+    def with_schema(self, schema: EagerSchema) -> Self:
         """Joined, subqueryload and selectinload eager loading.
 
         Useful for complex cases where you need to load
@@ -1592,10 +1582,9 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
 
     def _apply_eager_loading_options(
         self,
-        *paths: InstrumentedAttribute[Any]
-        | tuple[InstrumentedAttribute[Any], bool],
+        *paths: EagerLoadPath,
         joined: bool = False,
-        model: type[_T] | None = None,
+        model: type[T] | None = None,
     ) -> Self:
         """Applies the eager loading options from the given paths.
 
@@ -1609,13 +1598,16 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
 
         The ``joined`` flag is used for joined eager loading.
 
+        If path is a tuple, the second element is used to determine
+        whether to use selectin or, if ``joined`` is True, inner join.
+
         Parameters
         ----------
-        *paths : InstrumentedAttribute[Any] | tuple[InstrumentedAttribute[Any], bool]
+        *paths : EagerLoadPath
             Eager loading paths.
         joined : bool, optional
             Whether to use joined eager loading, by default False.
-        model : type[_T] | None, optional
+        model : type[T] | None, optional
             If given, checks that each path belongs to this model,
             by default None.
 
@@ -1635,35 +1627,28 @@ class AsyncQuery(SessionMixin, SmartQueryMixin, Generic[_T]):
         for path in paths:
             # if path is like (User.comments, True)
             if isinstance(path, tuple):
-                if not isinstance(path[1], bool):
+                attr, use_selectin = path
+                if not isinstance(use_selectin, bool):
                     raise TypeError(
-                        f"expected boolean for second element of tuple, "
-                        f"got {type(path[1])}: '{path[1]}'"
+                        f'expected boolean for second element of tuple, '
+                        f'got {type(use_selectin).__name__}: {use_selectin!r}'
                     )
 
-                # raise error if, i.e., model is User
-                # and path is Post.comments
-                if model and path[0].class_ != model:
-                    raise RelationError(path[0].key, model.__name__)
-
-                if joined:
-                    options.append(joinedload(path[0], innerjoin=path[1]))
-                    continue
-
-                options.append(
-                    selectinload(path[0]) if path[1] else subqueryload(path[0])
-                )
-
             # simple paths like User.posts, User.comments
             else:
-                if model and path.class_ != model:
-                    raise RelationError(path.key, model.__name__)
+                attr, use_selectin = path, False  # subqueryload by default
 
-                if joined:
-                    options.append(joinedload(path))
-                    continue
+            # raise error if, i.e., model is User
+            # and path is Post.comments
+            if model and attr.class_ != model:
+                raise RelationError(attr.key, model.__name__)
 
-                options.append(subqueryload(path))
+            if joined:
+                options.append(joinedload(attr, innerjoin=use_selectin))
+            else:
+                options.append(
+                    selectinload(attr) if use_selectin else subqueryload(attr)
+                )
 
         return self.options(*options)
 
